@@ -109,6 +109,11 @@ class SymbolPath(object):
         super(SymbolPath, self).__init__()
         self.__painterpath = painterpath
         self.__isfilled = isfilled
+        if isfilled:
+            try:
+                self.__painterpath = painterpath.simplified()
+            except:
+                pass
 
     def painterPath(self):
         '''
@@ -305,68 +310,171 @@ class CmndHelperPQ(object):
             pass
         return mypen
 
-    def getSymbolFromCmnd(self, symbol):
+    def getSymbolFromCmnd(self, symbolinfo):
         '''
-        Returns a SymbolPath of the specified symbol.
-        Recognized symbols are:
-            '.' (period): filled circle
-            'o' (lowercase oh): unfilled circle
-            '+': plus mark
-            'x' (lowercase ex): x mark
-            '*': asterisk
-            '^': triangle
-            "#": square
+        Returns the SymbolPath for the symbol described in symbolinfo, 
+        which can either be a string or a dictionary.  
 
-        The path is drawn for a 100 x 100 unit square where
-        the origin is in the center of the square.
+        If symbolinfo is a string, it should be the name of a symbol that 
+        has already been defined, either as a pre-defined symbol or from 
+        a previous symbol definition.
+
+        Current pre-defined symbol names are ones involving circles:
+            'dot': very small filled circle
+            'dotex': very small filled circle and outer lines of an ex mark
+            'dotplus': very small filled circle and outer lines of a plus mark
+            'circle': unfilled circle
+            'circfill': normal-sized filled circle
+            'circex': small unfilled circle and outer lines of an ex mark
+            'circplus': small unfilled circle and outer lines of a plus mark
+
+        If symbolinfo is a dictionary, the following key/value pairs are 
+        recognized:
+            'name' : (string) symbol name (required)
+            'pts'  : (sequence of pairs of floats) vertex coordinates
+            'fill' : (bool) color-fill symbol?
+
+        If 'pts' is given, the value is coordinates that define the symbol 
+        as multiline subpaths in a [-50,50] square for typical size.  The 
+        location of the point this symbol represents will be at the center 
+        of the square.  A coordinate outside [-100,100] will terminate the 
+        current subpath, and the next valid coordinate will start a new subpath. 
+        This definition will replace an existing symbol with the given name.
+
+        If 'pts' is not given, the symbol must already be defined, either as 
+        a pre-defined symbol (see above) or from a previous symbol definition.
+
+        Raises:
+             TypeError  - if symbolinfo is neither a string nor a dictionary
+             KeyError   - if symbolinfo is a dictionary and 
+                          the key 'name' is not given 
+             ValueError - if there are problems generating the symbol
         '''
-        # check if this symbol has already been created
-        try:
-            sympath = self.__symbolpaths[symbol]
-            return sympath
-        except KeyError:
-            pass
-        # new symbol - create a SymbolPath for it
-        if symbol == '.':
-            path = QPainterPath()
-            path.addEllipse(-10.0, -10.0, 20.0, 20.0)
-            sympath = SymbolPath(path, True)
-        elif symbol == 'o':
-            path = QPainterPath()
-            path.addEllipse(-40.0, -40.0, 80.0, 80.0)
-            sympath = SymbolPath(path, False)
-        elif symbol == 'x':
-            path = QPainterPath( QPointF(-30.0, -30.0) )
-            path.lineTo( 30.0,  30.0)
-            path.moveTo(-30.0,  30.0)
-            path.lineTo( 30.0, -30.0)
-            sympath = SymbolPath(path, False)
-        elif symbol == '+':
-            path = QPainterPath( QPointF(0.0, -40.0) )
-            path.lineTo(  0.0, 40.0)
-            path.moveTo(-40.0,  0.0)
-            path.lineTo( 40.0,  0.0)
-            sympath = SymbolPath(path, False)
-        elif symbol == '*':
-            path = QPainterPath( QPointF(0.0, -40.0) )
-            path.lineTo(  0.0,    40.0)
-            path.moveTo(-34.641, -20.0)
-            path.lineTo( 34.641,  20.0)
-            path.moveTo(-34.641,  20.0)
-            path.lineTo( 34.641, -20.0)
-            sympath = SymbolPath(path, False)
-        elif symbol == '^':
-            path = QPainterPath( QPointF(-40.0, 30.0) )
-            path.lineTo( 0.0, -39.282)
-            path.lineTo(40.0,  30.0)
-            path.closeSubpath()
-            sympath = SymbolPath(path, False)
-        elif symbol == '#':
-            path = QPainterPath()
-            path.addRect(-35.0, -35.0, 70.0, 70.0)
-            sympath = SymbolPath(path, False)
+        # get the information about the symbol
+        if isinstance(symbolinfo, str):
+            symbol = symbolinfo
+            pts = None
+            fill = False
+        elif isinstance(symbolinfo, dict):
+            symbol = symbolinfo['name']
+            pts = symbolinfo.get('pts', None)
+            fill = symbolinfo.get('fill', False)
         else:
-            raise ValueError("Unrecognized symbol '%s'" % str(symbol))
+            raise TypeError('symbolinfo must either be a dictionary or a string')
+
+        if pts is None:
+            # no path given; check if already defined
+            sympath = self.__symbolpaths.get(symbol)
+            if sympath is not None:
+                return sympath
+            # symbol not defined - if well known, create a SymbolPath for it
+            if symbol == 'dot':
+                path = QPainterPath()
+                path.addEllipse(-10.0, -10.0, 20.0, 20.0)
+                sympath = SymbolPath(path, True)
+            elif symbol == 'dotplus':
+                path = QPainterPath()
+                path.addEllipse(-10.0, -10.0, 20.0, 20.0)
+                # filled path, so need to draw "lines" as rectangles
+                path.addRect( -4.0, -50.0,  8.0, 24.0)
+                path.addRect( -4.0,  26.0,  8.0, 24.0)
+                path.addRect(-50.0,  -4.0, 24.0,  8.0)
+                path.addRect( 26.0,  -4.0, 24.0,  8.0)
+                sympath = SymbolPath(path, True)
+            elif symbol == 'dotex':
+                path = QPainterPath()
+                path.addEllipse(-10.0, -10.0, 20.0, 20.0)
+                # filled path, so need to draw "lines" as rectangles
+                path.moveTo(-38.18, -32.53)
+                path.lineTo(-32.53, -38.18)
+                path.lineTo(-15.56, -21.21)
+                path.lineTo(-21.21, -15.56)
+                # moveTo adds an implicit closeSubpath in QPainterPath
+                path.moveTo(-38.18,  32.53)
+                path.lineTo(-32.53,  38.18)
+                path.lineTo(-15.56,  21.21)
+                path.lineTo(-21.21,  15.56)
+                # moveTo adds an implicit closeSubpath in QPainterPath
+                path.moveTo( 38.18, -32.53)
+                path.lineTo( 32.53, -38.18)
+                path.lineTo( 15.56, -21.21)
+                path.lineTo( 21.21, -15.56)
+                # moveTo adds an implicit closeSubpath in QPainterPath
+                path.moveTo( 38.18,  32.53)
+                path.lineTo( 32.53,  38.18)
+                path.lineTo( 15.56,  21.21)
+                path.lineTo( 21.21,  15.56)
+                # Qt closes the subpath automatically
+                sympath = SymbolPath(path, True)
+            elif symbol == 'circle':
+                path = QPainterPath()
+                path.addEllipse(-35.0, -35.0, 70.0, 70.0)
+                sympath = SymbolPath(path, False)
+            elif symbol == 'circfill':
+                path = QPainterPath()
+                path.addEllipse(-39.0, -39.0, 78.0, 78.0)
+                sympath = SymbolPath(path, True)
+            elif symbol == 'circplus':
+                path = QPainterPath()
+                path.addEllipse(-20.0, -20.0, 40.0, 40.0)
+                # not a filled path, so just draw the lines
+                path.moveTo(  0.0, -50.0)
+                path.lineTo(  0.0, -20.0)
+                path.moveTo(  0.0,  50.0)
+                path.lineTo(  0.0,  20.0)
+                path.moveTo(-50.0,   0.0)
+                path.lineTo(-20.0,   0.0)
+                path.moveTo( 50.0,   0.0)
+                path.lineTo( 20.0,   0.0)
+                sympath = SymbolPath(path, False)
+            elif symbol == 'circex':
+                path = QPainterPath()
+                path.addEllipse(-20.0, -20.0, 40.0, 40.0)
+                # not a filled path, so just draw the lines
+                path.moveTo(-35.35, -35.35)
+                path.lineTo(-14.15, -14.15)
+                path.moveTo(-35.35,  35.35)
+                path.lineTo(-14.15,  14.15)
+                path.moveTo( 35.35, -35.35)
+                path.lineTo( 14.15, -14.15)
+                path.moveTo( 35.35,  35.35)
+                path.lineTo( 14.15,  14.15)
+                sympath = SymbolPath(path, False)
+            else:
+                raise ValueError("Unknown symbol '%s'" % str(symbol))
+        else:
+            # define (or redefine) a symbol from the given path
+            try:
+                coords = [ [ float(val) for val in coord ] for coord in pts ]
+                if not coords:
+                    raise ValueError
+                for crd in coords:
+                    if len(crd) != 2:
+                        raise ValueError
+            except Exception:
+                raise ValueError('pts, if given, must be a sequence of pairs of numbers')
+            path = QPainterPath()
+            somethingdrawn = False
+            newstart = True
+            for (xval, yval) in coords:
+                # flip so positive y is up
+                yval *= -1.0
+                if (xval < -100.0) or (xval > 100.0) or (yval < -100.0) or (yval > 100.0):
+                    # end the current subpath
+                    newstart = True
+                elif newstart:
+                    # start a new subpath; moveTo adds an implicit closeSubpath in QPainterPath
+                    path.moveTo(xval, yval)
+                    newstart = False
+                else:
+                    # continue the current subpath
+                    path.lineTo(xval, yval)
+                    somethingdrawn = True
+            if not somethingdrawn:
+                del path
+                raise ValueError('symbol definition does not contain any drawn lines')
+            # Qt closes the (sub)path automatically
+            sympath = SymbolPath(path, fill)
         # save and return the SymbolPath
         self.__symbolpaths[symbol] = sympath
         return sympath
