@@ -225,8 +225,13 @@ class PipedViewerPQ(QMainWindow):
         '''
         self.__timer.stop()
         self.__cmndpipe.close()
-        self.__rspdpipe.send(WINDOW_CLOSED_MESSAGE)
-        self.__rspdpipe.close()
+        try:
+            try:
+                self.__rspdpipe.send(WINDOW_CLOSED_MESSAGE)
+            finally:
+                self.__rspdpipe.close()
+        except Exception:
+            pass
         event.accept()
 
     def exitViewer(self):
@@ -952,9 +957,11 @@ class PipedViewerPQ(QMainWindow):
                 # of commands waiting in the queue.
                 if (time.clock() - starttime) > 0.050:
                     break
+        except EOFError:
+            # Assume PyFerret has shut down
+            self.exitViewer()
         except Exception:
-            # EOFError should never arise from recv since
-            # the call is after poll returns True
+            # Some problem, but presumably still functional
             (exctype, excval) = sys.exc_info()[:2]
             try:
                 if excval:
@@ -963,12 +970,11 @@ class PipedViewerPQ(QMainWindow):
                     self.__rspdpipe.send("**ERROR %s" % str(exctype))
             except Exception:
                 pass
-            self.exitViewer()
 
     def processCommand(self, cmnd):
         '''
         Examine the action of cmnd and call the appropriate
-        method to deal with this command.  Raises a KeyError
+        method to deal with this command.  Raises a ValueError
         if the "action" key is missing.
         '''
         try:
@@ -1050,6 +1056,11 @@ class PipedViewerPQ(QMainWindow):
             self.endSegment(True)
         elif cmndact == "deleteSegment":
             self.deleteSegment(cmnd["segid"])
+        elif cmndact == "createSymbol":
+            # Define this symbol in self.__symbolpaths
+            sympath = self.__helper.getSymbolFromCmnd(cmnd)
+            # The name is now all that is needed to use this symbol
+            self.__rspdpipe.send(cmnd['name'])
         elif cmndact == "drawMultiline":
             self.drawMultiline(cmnd)
         elif cmndact == "drawPoints":
@@ -1269,12 +1280,13 @@ class PipedViewerPQ(QMainWindow):
 
         Recognized keys from cmnd:
             "points": point centers as a list of (x,y) coordinates
-            "symbol": name of the symbol to use
-                    (see CmndHelperPQ.getSymbolFromCmnd)
-            "size": size of the symbol in points (1/72 inches); possibly
-                    further scaled by the width scaling factor
+            "symbol": symbol to use (see CmndHelperPQ.getSymbolFromCmnd)
+            "size": size of the symbol in points (1/72 inches) before
+                    scaling by the width scaling factor
             "color": color name or 24-bit RGB integer value (eg, 0xFF0088)
             "alpha": alpha value from 0 (transparent) to 255 (opaque)
+            "highlight": dictionary of "color" and "alpha" (as above)
+                     for filled symbol outline color; not outlined if omitted
 
         The coordinates are device coordinates from the upper left corner.
 
@@ -1283,6 +1295,11 @@ class PipedViewerPQ(QMainWindow):
         '''
         ptcoords = cmnd["points"]
         ptsize = cmnd["size"]
+        ptsize = cmnd["size"]
+        try:
+            highlight = self.__helper.getColorFromCmnd(cmnd["highlight"])
+        except KeyError:
+            highlight = None
         sympath = self.__helper.getSymbolFromCmnd(cmnd["symbol"])
         self.__activepainter.setRenderHint(QPainter.Antialiasing,
                                            self.__antialias)
@@ -1293,14 +1310,20 @@ class PipedViewerPQ(QMainWindow):
             mybrush = QBrush(Qt.SolidPattern)
         if sympath.isFilled():
             self.__activepainter.setBrush(mybrush)
-            self.__activepainter.setPen(Qt.NoPen)
+            if highlight:
+                # highlighted filled plot - pen width is 4% of the width of the symbol
+                mybrush = QBrush(highlight, Qt.SolidPattern)
+                mypen = QPen(mybrush, 4.0, Qt.SolidLine, Qt.FlatCap, Qt.MiterJoin)
+                self.__activepainter.setPen(mypen)
+            else:
+                # filled plot without highlight - no pen, only brush
+                self.__activepainter.setPen(Qt.NoPen)
         else:
+            # stroked path - no brush, pen width is 8% of the width of the symbol, highlight is ignored
             self.__activepainter.setBrush(Qt.NoBrush)
-            # pen width is 15% of the width of the symbol
-            mypen = QPen(mybrush, 15.0, Qt.SolidLine,
-                         Qt.SquareCap, Qt.BevelJoin)
+            mypen = QPen(mybrush, 8.0, Qt.SolidLine, Qt.FlatCap, Qt.MiterJoin)
             self.__activepainter.setPen(mypen)
-        # Unmodified symbols are 100x100 pixels 
+        # typical symbols are 100x100 pixels 
         scalefactor = ptsize * self.widthScalingFactor() / 100.0
         if self.__maxsymbolwidth < 100.0 * scalefactor:
             self.__maxsymbolwidth = 100.0 * scalefactor
@@ -1535,7 +1558,7 @@ class PipedViewerPQProcess(Process):
         Create a Process that will produce a PipedViewerPQ
         attached to the given Pipes when run.
         '''
-        Process.__init__(self)
+        super(PipedViewerPQProcess,self).__init__(group=None, target=None, name='PipedViewerPQ')
         self.__cmndpipe = cmndpipe
         self.__rspdpipe = rspdpipe
         self.__app = None
@@ -1566,7 +1589,7 @@ class _CommandSubmitterPQ(QDialog):
         Create a QDialog with a single QPushButton for controlling
         the submission of commands from cmndlist to cmndpipe.
         '''
-        QDialog.__init__(self, parent)
+        super(_CommandSubmitterPQ,self).__init__(parent)
         self.__cmndlist = cmndlist
         self.__cmndpipe = cmndpipe
         self.__rspdpipe = rspdpipe
@@ -1653,6 +1676,18 @@ def _test_pipedviewerpq():
     drawcmnds.append( { "action":"endSegment" } )
     drawcmnds.append( { "action":"endView" } )
     drawcmnds.append( { "action":"show" } )
+    drawcmnds.append( { "action":"createSymbol",
+                        "name": "uptrifill",
+                        "pts": ( (-40.0, -30.0), (0.0, 40.0), (40.0, -30.0), (-40.0, -30.0), ),
+                        "fill": True } )
+    drawcmnds.append( { "action":"createSymbol",
+                        "name": "bararrow",
+                        "pts": ( (-50,50), (-10,10),
+                                 (-999, -999),
+                                 (50,0), (50,50), (0,50),
+                                 (-999, -999),
+                                 (0,-10), (20,-30), (10,-30), (10,-50), (-10,-50), (-10,-30), (-20,-30), (0,-10), ),
+                        "fill": False } )
     drawcmnds.append( { "action":"beginView",
                         "viewfracs":{"left":0.0, "right":1.0,
                                      "top":0.0, "bottom":1.0},
@@ -1663,7 +1698,7 @@ def _test_pipedviewerpq():
                                    (100, 250),
                                    (100, 350),
                                    (100, 450) ),
-                        "symbol":".",
+                        "symbol":"dot",
                         "size":20,
                         "color":"magenta" })
     drawcmnds.append( { "action":"drawPoints",
@@ -1672,7 +1707,7 @@ def _test_pipedviewerpq():
                                    (150, 250),
                                    (150, 350),
                                    (150, 450) ),
-                        "symbol":"o",
+                        "symbol":"circle",
                         "size":20,
                         "color":"magenta" })
     drawcmnds.append( { "action":"drawPoints",
@@ -1681,7 +1716,7 @@ def _test_pipedviewerpq():
                                    (200, 250),
                                    (200, 350),
                                    (200, 450) ),
-                        "symbol":"+",
+                        "symbol":"dotplus",
                         "size":20,
                         "color":"magenta" })
     drawcmnds.append( { "action":"drawPoints",
@@ -1690,7 +1725,7 @@ def _test_pipedviewerpq():
                                    (250, 250),
                                    (250, 350),
                                    (250, 450) ),
-                        "symbol":"x",
+                        "symbol":"circplus",
                         "size":20,
                         "color":"magenta" })
     drawcmnds.append( { "action":"drawPoints",
@@ -1699,7 +1734,7 @@ def _test_pipedviewerpq():
                                    (300, 250),
                                    (300, 350),
                                    (300, 450) ),
-                        "symbol":"*",
+                        "symbol":"dotex",
                         "size":20,
                         "color":"magenta" })
     drawcmnds.append( { "action":"drawPoints",
@@ -1708,7 +1743,7 @@ def _test_pipedviewerpq():
                                    (350, 250),
                                    (350, 350),
                                    (350, 450) ),
-                        "symbol":"^",
+                        "symbol":"circex",
                         "size":20,
                         "color":"magenta" })
     drawcmnds.append( { "action":"drawPoints",
@@ -1717,7 +1752,16 @@ def _test_pipedviewerpq():
                                    (400, 250),
                                    (400, 350),
                                    (400, 450) ),
-                        "symbol":"#",
+                        "symbol":"uptrifill",
+                        "size":20,
+                        "color":"magenta" })
+    drawcmnds.append( { "action":"drawPoints",
+                        "points":( (450,  50),
+                                   (450, 150),
+                                   (450, 250),
+                                   (450, 350),
+                                   (450, 450) ),
+                        "symbol":"bararrow",
                         "size":20,
                         "color":"magenta" })
     drawcmnds.append( { "action":"drawMultiline",
